@@ -1,16 +1,19 @@
 from functools import singledispatch
-from typing import Tuple
+from typing import TYPE_CHECKING, Sequence
 
 import aesara.tensor as at
 import aesara.tensor.random.basic as arb
 import numpy as np
-from aesara.graph.op import Op
 from aesara.raise_op import CheckAndRaise
 from aesara.tensor.slinalg import Cholesky, solve_triangular
-from aesara.tensor.var import TensorVariable
 
+from aeppl.abstract import ValuedVariable
 from aeppl.dists import DiracDelta, DiscreteMarkovChainFactory
 
+if TYPE_CHECKING:
+    from aesara.graph.op import Op
+    from aesara.tensor.var import TensorVariable
+
 
 class ParameterValueError(ValueError):
     """Exception for invalid parameters values in logprob graphs"""
@@ -50,19 +53,20 @@ def xlogy0(m, x):
     return at.switch(at.eq(x, 0), at.switch(at.eq(m, 0), 0.0, -np.inf), m * at.log(x))
 
 
-def logprob(rv_var, *rv_values, **kwargs):
-    """Create a graph for the log-probability of a ``RandomVariable``."""
-    logprob = _logprob(rv_var.owner.op, rv_values, *rv_var.owner.inputs, **kwargs)
+def logprob(rv_val, *rv_values, **kwargs):
+    """Create a graph for the log-probability of a measurable variable."""
 
-    for rv_var in rv_values:
-        if rv_var.name:
-            logprob.name = f"{rv_var.name}_logprob"
+    logprob = _logprob(rv_val.owner.op, rv_values, *rv_val.owner.inputs, **kwargs)
+
+    for rv_val in rv_values:
+        if rv_val.name:
+            logprob.name = f"{rv_val.name}_logprob"
 
     return logprob
 
 
 def logcdf(rv_var, rv_value, **kwargs):
-    """Create a graph for the logcdf of a ``RandomVariable``."""
+    """Create a graph for the log-CDF of a measurable variable."""
     logcdf = _logcdf(
         rv_var.owner.op, rv_value, *rv_var.owner.inputs, name=rv_var.name, **kwargs
     )
@@ -74,7 +78,7 @@ def logcdf(rv_var, rv_value, **kwargs):
 
 
 def icdf(rv, value, **kwargs):
-    """Create a graph for the inverse CDF of a `RandomVariable`."""
+    """Create a graph for the inverse CDF of a measurable variable."""
     rv_icdf = _icdf(rv.owner.op, value, *rv.owner.inputs, **kwargs)
     if rv.name:
         rv_icdf.name = f"{rv.name}_icdf"
@@ -83,52 +87,43 @@ def icdf(rv, value, **kwargs):
 
 @singledispatch
 def _logprob(
-    op: Op,
-    values: Tuple[TensorVariable],
-    *inputs: TensorVariable,
+    op: "Op",
+    values: Sequence["TensorVariable"],
+    *inputs: "TensorVariable",
     **kwargs,
 ):
-    """Create a graph for the log-density/mass of a ``RandomVariable``.
-
-    This function dispatches on the type of ``op``, which should be a subclass
-    of ``RandomVariable``.  If you want to implement new density/mass graphs
-    for a ``RandomVariable``, register a new function on this dispatcher.
-
-    """
+    """Create a graph for the log-density/mass of a measurable variable."""
     raise NotImplementedError(f"Logprob method not implemented for {op}")
 
 
 @singledispatch
 def _logcdf(
-    op: Op,
-    value: TensorVariable,
-    *inputs: TensorVariable,
+    op: "Op",
+    value: "TensorVariable",
+    *inputs: "TensorVariable",
     **kwargs,
 ):
-    """Create a graph for the logcdf of a ``RandomVariable``.
-
-    This function dispatches on the type of ``op``, which should be a subclass
-    of ``RandomVariable``.  If you want to implement new logcdf graphs
-    for a ``RandomVariable``, register a new function on this dispatcher.
-    """
+    """Create a graph for the log-CDF of a measurable variable."""
     raise NotImplementedError(f"Logcdf method not implemented for {op}")
 
 
 @singledispatch
 def _icdf(
-    op: Op,
-    value: TensorVariable,
-    *inputs: TensorVariable,
+    op: "Op",
+    value: "TensorVariable",
+    *inputs: "TensorVariable",
     **kwargs,
 ):
-    """Create a graph for the inverse CDF of a `RandomVariable`.
-
-    This function dispatches on the type of `op`, which should be a subclass
-    of `RandomVariable`.
-    """
+    """Create a graph for the inverse CDF of a measurable variable."""
     raise NotImplementedError(f"icdf not implemented for {op}")
 
 
+@_logprob.register(ValuedVariable)
+def valued_variable_logprob(op: ValuedVariable, values, *inputs, **kwargs):
+    measurable_input = inputs[0]
+    return logprob(measurable_input, *values, **kwargs)
+
+
 @_logprob.register(arb.UniformRV)
 def uniform_logprob(op, values, *inputs, **kwargs):
     (value,) = values
@@ -534,6 +529,20 @@ def hypergeometric_logprob(op, values, *inputs, **kwargs):
     return res
 
 
+@_logprob.register(arb.StudentTRV)
+def studentt_logprob(op, values, *inputs, **kwargs):
+    (value,) = values
+    df, loc, scale = inputs[3:]
+    res = (
+        at.gammaln((df + 1.0) / 2.0)
+        - at.gammaln(df / 2.0)
+        - 0.5 * at.log(np.pi * df * scale**2)
+        - (df + 1.0) / 2.0 * at.log1p(((value - loc) / scale) ** 2 / df)
+    )
+    res = CheckParameterValue("scale >= 0")(res, at.all(at.ge(scale, 0.0)))
+    return res
+
+
 @_logprob.register(arb.CategoricalRV)
 def categorical_logprob(op, values, *inputs, **kwargs):
     (value,) = values
@@ -555,7 +564,7 @@ def categorical_logprob(op, values, *inputs, **kwargs):
             )
         )
         # FIXME: `take_along_axis` drops a broadcastable dimension
-        # when `value.broadcastable == p.broadcastable == (True, True, False)`.
+        # when `value.type.shape == p.type.shape == (1, 1, None)`.
     else:
         res = at.log(p[value])
 
