@@ -101,7 +101,7 @@ class RandomVariablePrinter:
                 "not the result of a RandomVariable operation" % self.name
             )
 
-        op_name = self.name or getattr(node.op, "print_name", None)
+        op_name = self.name or getattr(node.op, "_print_name", None)
         op_name = op_name or getattr(node.op, "name", None)
 
         if op_name is None:  # pragma: no cover
@@ -130,7 +130,6 @@ class RandomVariablePrinter:
             dummy_out.orig_var = output
 
             var_name = pprinter.process(dummy_out, pstate)
-
             if output_latex:
                 dist_format = "%s \\sim %s\\left(%s\\right)"
             else:
@@ -138,6 +137,7 @@ class RandomVariablePrinter:
 
             # Get the shape info for our dummy symbol, if available,
             # and append it to the distribution definition.
+            # TODO: Propagate this change upstream in Aesara's pretty printer.
             if "shape_strings" in preamble_dict:
                 shape_info_str = preamble_dict["shape_strings"].pop(dummy_out)
                 shape_info_str = shape_info_str.lstrip(var_name)
@@ -152,6 +152,17 @@ class RandomVariablePrinter:
                 for i, p in enumerate(dist_params)
             ]
 
+            # We remove trailing zeros and limit the number of decimals
+            # on floats
+            formatted_params = []
+            for i, p in enumerate(dist_params):
+                f_param = self.process_param(i, pprinter.process(p, pstate), pstate)
+                try:
+                    f_param = f"{float(f_param):2g}".strip()
+                except ValueError:
+                    pass
+                formatted_params.append(f_param)
+
             dist_def_str = dist_format % (
                 var_name,
                 op_name,
@@ -406,7 +417,8 @@ class PreamblePPrinter(PPrinter):
     -------
     >>> import aesara.tensor as at
     >>> from aeppl.printing import pprint
-    >>> X_rv = at.random.normal(at.scalar('\\mu'), at.scalar('\\sigma'), name='X')
+    >>> srng = at.random.RandomStream()
+    >>> X_rv = srng.normal(at.scalar('\\mu'), at.scalar('\\sigma'), name='X')
     >>> print(pprint(X_rv))
     \\mu in R
     \\sigma in R
@@ -527,7 +539,6 @@ class PreamblePPrinter(PPrinter):
         # categories when the joined line is too long.
         preamble_lines = []
         for v in pstate.preamble_dict.values():
-
             if isinstance(v, Mapping):
                 v = list(v.values())
 
@@ -550,9 +561,12 @@ class PreamblePPrinter(PPrinter):
             preamble_lines.append(newline_str.join(comma_str.join(z) for z in v_new))
 
         if preamble_lines and latex_out:
-            preamble_body = newline_str.join(preamble_lines)
-            preamble_str = f"\\begin{{gathered}}\n{textwrap.indent(preamble_body, indent_str)}\n\\end{{gathered}}"
-            res = newline_str.join([preamble_str] + body_strs)
+            preamble_body = newline_str.join(preamble_lines + body_strs)
+            preamble_str = (
+                f"\\begin{{gathered}}\n{textwrap.indent(preamble_body, indent_str)}"
+                f"\n\\end{{gathered}}"
+            )
+            res = newline_str.join([preamble_str])
         else:
             res = newline_str.join(preamble_lines + body_strs)
 
@@ -655,6 +669,6 @@ latex_pprint.assign(at.invert, PatternPrinter(("\\lnot %(0)s", -1000)))
 
 latex_pprint.assign(_dot, OperatorPrinter("\\;", -1, "left"))
 latex_pprint.assign(at.mul, OperatorPrinter("\\odot", -1, "either"))
-latex_pprint.assign(at.true_div, PatternPrinter(("\\frac{%(0)s}{%(1)s}", -1000)))
+latex_pprint.assign(at.true_divide, PatternPrinter(("\\frac{%(0)s}{%(1)s}", -1000)))
 latex_pprint.assign(at.sqrt, PatternPrinter(("\\sqrt{%(0)s}", -1000)))
 latex_pprint.assign(at.pow, PatternPrinter(("{%(0)s}^{%(1)s}", -1000)))
