@@ -10,7 +10,7 @@ from aesara.graph.features import AlreadyThere, Feature
 from aesara.graph.fg import FunctionGraph
 from aesara.graph.op import Op
 from aesara.graph.rewriting.basic import GraphRewriter, in2out, node_rewriter
-from aesara.tensor.math import add, exp, log, mul, reciprocal, sub, true_div
+from aesara.tensor.math import add, exp, log, mul, reciprocal, sub, true_divide
 from aesara.tensor.rewriting.basic import register_useless
 from aesara.tensor.var import TensorVariable
 from typing_extensions import Protocol
@@ -24,7 +24,7 @@ from aeppl.abstract import (
     valued_variable,
 )
 from aeppl.logprob import _logprob, logprob
-from aeppl.rewriting import measurable_ir_rewrites_db
+from aeppl.rewriting import ir_cleanup_db, measurable_ir_rewrites_db
 
 if TYPE_CHECKING:
     from aesara.graph.rewriting.basic import NodeRewriter
@@ -75,6 +75,7 @@ class TransformedVariable(Op):
 
     """
 
+    __props__ = ()
     view_map = {0: [0]}
 
     def make_node(self, tran_value: TensorVariable, value: TensorVariable):
@@ -101,8 +102,12 @@ transformed_variable = TransformedVariable()
 @register_useless
 @node_rewriter([TransformedVariable])
 def remove_TransformedVariables(fgraph, node):
-    if isinstance(node.op, TransformedVariable):
-        return [node.inputs[0]]
+    return [node.inputs[0]]
+
+
+ir_cleanup_db.register(
+    "remove-TransformedVariables", in2out(remove_TransformedVariables), "basic"
+)
 
 
 class RVTransform(abc.ABC):
@@ -381,9 +386,9 @@ def measurable_elemwise_logprob(
 
 
 @register_measurable_ir
-@node_rewriter([true_div])
-def measurable_true_div(fgraph, node):
-    r"""Rewrite a `true_div` node to a `MeasurableVariable`.
+@node_rewriter([true_divide])
+def measurable_true_divide(fgraph, node):
+    r"""Rewrite a `true_divide` node to a `MeasurableVariable`.
 
     TODO FIXME: We need update/clarify the canonicalization situation so that
     these can be reliably rewritten as products of reciprocals.
